/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/
#include "project.h"
#include <stdio.h>
// #include "vt100.h"
#include "i2c_utils.h"
#include "TFT.h"
#include "Terminal6x8.h"
#include "KXTJ2.h"
#include "maze.h"

#define DIR_STAY  0
#define DIR_UP    1
#define DIR_DOWN  2
#define DIR_RIGHT 3 
#define DIR_LEFT  4

#define MMA8451_I2C_ADDRESS (0x1D<<1)

typedef struct _pos {
    int x ;
    int y ;
} pos_type ;

float threshold = 0.2 ;

int data_ready = 1 ; /* 1 for test 0 ; */

char str[128] ;

/** Check if two pos_type values are same
 * @param pos_type a
 * @param pos_type b
 * @returns if a and b are same position
 */
int isSame(pos_type a, pos_type b)
{
    return((a.x == b.x)&&(a.y == b.y)) ;
}


/** Draw the maze defined in the "maze.h"
 * @param pos_type *current actually the start point
 * @param pos_type *goal the position of the goal
 * @note those params are actually returned by this function
 */
void drawTFTMaze(pos_type *current, pos_type *goal)
{
    int x, y, i, j ;
    TFT_BusEnable(1) ;
    
    TFT_BackLight(0) ;
    for (j = 0 ; j < MAZE_H ; j++ ) {
        for (i = 0 ; i < MAZE_W ; i++ ) {
            x = i * 10 ;
            y = j * 10 ;
            switch(maze[j][i]) {
            case 0: // path
                TFT_fillrect(x, y, x+9, y+9, Black) ;
                break ;
            case 1: // wall
                TFT_fillrect(x, y, x+9, y+9, DarkGrey) ;
                break ;
            case 2: // Start point
                TFT_fillcircle(x+5, y+4, 4, Green) ;
                current->x = i ;
                current->y = j ;
                break ;
            case 3: // Goal
                TFT_fillcircle(x+5, y+4, 4, Red) ;
                goal->x = i ;
                goal->y = j ;
                break ;
            default: // should not be here
                break ;
            }
        }
    }
    TFT_BusEnable(0) ;
    
    TFT_BackLight(1) ;
}

/** Filter out too little move 
 * @param float in returned value from the acc
 * @returns float result filtered value of in
 */
float filterVal(float in)
{
    float result = 0.0 ;
    if ((-threshold > in)||(in > threshold)) {
        result = in ;
    }
    return( result ) ;
}

/** Decide which direction to go
 * @param float res[] acc value of x, y 
 * @returns int direction to move 
 */
int getDirection(float res[])
{
    float dx, dy ;
    int direction = DIR_STAY ;
    dx = filterVal(res[0]) ;
    dy = filterVal(res[1]) ;

    if ((dx*dx) > (dy*dy)) { // holizontal move
        if (dx > 0.0) {
            direction = DIR_LEFT ;
        } else if (dx < 0.0) {
            direction = DIR_RIGHT ;
        }
    } else { // vertical move
        if (dy > 0.0) {
            direction = DIR_DOWN ;
        } else if (dy < 0.0) {
            direction = DIR_UP ;
        }
    }
    return(direction) ;
}

/** Get next positon to move to
 * @param pos_type current where we are now
 * @param int direction which way we'd like to move
 * @returns the candidate positon for the next move
 */
pos_type getNext(pos_type current, int direction) 
{
    pos_type next = current ;
    switch(direction) {
    case DIR_STAY: 
        break ;
    case DIR_UP: 
        if (next.y > 0) { 
            next.y-- ; 
        }  
        break ;
    case DIR_DOWN: 
        if (next.y < (MAZE_H - 1)) { 
            next.y++ ; 
        } 
        break ;
    case DIR_RIGHT: 
        if (next.x < (MAZE_W - 1)) { 
            next.x++ ; 
        } 
        break ;
    case DIR_LEFT: 
        if (next.x > 0) { 
            next.x-- ; 
        } 
        break ;
    default: 
        break ;
    }
    return( next ) ;
}

/** Notice of the goal
 */
void showGoal(void)
{
    int x0, y0, x1, y1 ;
    x0 = ((MAZE_W/2)-4) * 10 ;
    y0 = ((MAZE_H/2)-1) * 10 ;
    x1 = ((MAZE_W/2)+4) * 10 ;
    y1 = ((MAZE_H/2)+1) * 10 ;

    TFT_BusEnable(1) ;
    TFT_fillrect(x0, y0, x1, y1, Blue) ;
    TFT_rect(x0,y0,x1,y1, Yellow) ;
    TFT_foreground(Red) ;
    TFT_background(Blue) ;
    x0 = ((MAZE_W/2) - 2) * 10 ;
    y0 = (MAZE_H/2) * 10 - 3 ;
    TFT_locate(x0, y0) ;
    TFT_putstr("G O A L") ;
    TFT_BusEnable(0) ;
}

/** Check if we can move to the next position
 * @param pos_type next the position we'd like to move next
 * @returns if the position is empty (movable)
 */
int checkMove(pos_type next) 
{
    int result = 0 ;
    
    switch(maze[next.y][next.x]) {
    case POS_PATH:
    case POS_GOAL:
        result = 1 ;
        break ;
    case POS_START:
    case POS_WALL:
    default:
        result = 0 ;
        break ;
    } 
    return( result ) ;
}

CY_ISR(KXTJ2_INT_isr)
{
    data_ready = 1 ;
}

void init_hardware(void)
{
    CyGlobalIntEnable; /* Enable global interrupts. */

    TFT_CS_Write(1) ;
    TFT_DC_Write(0) ;
    TFT_BL_Write(1) ;
    
    SPI_Start() ;
    I2C_Start() ;
    
    TFT_Init() ;
    TFT_BusEnable(1) ;
    TFT_foreground(White) ;
    TFT_background(Black) ;
    TFT_set_orientation(0) ;
    TFT_cls() ;
    TFT_set_font(Terminal6x8) ;
    TFT_BL_Write(1) ;
    TFT_BusEnable(0) ;
    
    KXTJ2_init() ;
    
    // associate KXTJ2_INT for KXTJ2_INT_isr .. but how?
}

int main(void)
{
    float res[3] ;
    pos_type current, next, goal ;
    int direction = DIR_STAY ;
    int x, y ;

    init_hardware() ;
    
    drawTFTMaze(&current, &goal) ;
        
    for(;;) {
        KXTJ2_getAllAxis(res) ;
        direction = getDirection(res) ;
        next = getNext(current, direction) ;
        if ((!isSame(current, next)) && checkMove(next)) {
           TFT_BusEnable(1) ;
           x = current.x * 10 ;
           y = current.y * 10 ;
           TFT_fillrect(x, y, x+9, y+9, Black) ;
           x = next.x * 10 ;
           y = next.y * 10 ;
           TFT_fillcircle(x+5, y+5, 4, Green) ; 
           TFT_BusEnable(0) ;
           current = next ;
           if (isSame(next, goal)) {
             break ; // goal in!
           }
        }
        CyDelay(100) ;
    }
    showGoal() ;
    for (;;) {
        // wait for ever for reset
    }
}

/* [] END OF FILE */
