/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/
#include "project.h"
#include <string.h>
#include "vt100.h"
#include "i2c_utils.h"
#include "KXTJ2.h"

#define REG_XOUT_L           0x06
#define REG_XOUT_H           0x07
#define REG_YOUT_L           0x08
#define REG_YOUT_H           0x09
#define REG_ZOUT_L           0x0A
#define REG_ZOUT_H           0x0B
#define REG_DCST_RESP        0x0C
#define REG_WHO_AM_I         0x0F
#define REG_INT_SOURCE1      0x16
#define REG_INT_SOURCE2      0x17
#define REG_STATUS_REG       0x18
#define REG_INT_REL          0x1A
#define REG_CTRL_REG1        0x1B
#define REG_CTRL_REG2        0x1D
#define REG_INT_CTRL_REG1    0x1E
#define REG_INT_CTRL_REG2    0x1F
#define REG_DATA_CTRL_REG    0x21
#define REG_WAKEUP_TIMER     0x29
#define REG_SELF_TEST        0x3A
#define REG_WAKEUP_THRESHOLD 0x6A

// #define I2C_KXTJ2_ADDRESS 0x0E
#define I2C_KXTJ2_ADDRESS 0x0F
extern char str[] ;

void    KXTJ2_reboot(void) 
{
    i2c_set_slave_address(I2C_KXTJ2_ADDRESS) ;
}

/**
 * KXTJ2_init
 * Note: 
 * Although in the datasheet and application note AN038
 * it is written to set PC1 = 0 before setting register(s)
 * as of 23-Mar-2018, if I write 00 to REG_CTRL_REG1
 * the device won't come back operation mode even if 0x80 
 * is written later. Registers can be written but data 
 * does not change.
 */
void KXTJ2_init(void) 
{
    uint8_t data[1] ;
 
    i2c_set_slave_address(I2C_KXTJ2_ADDRESS) ;
    
    data[0] = 0x80 ; // Run!! 
//    i2c_writeRegs(REG_CTRL_REG1, data, 1) ;
    i2c_writeReg(REG_CTRL_REG1, data[0]) ;

//    data[0] = 0x00 ; /* stand-by mode (PC1 = 0) */
//    i2c_writeReg(REG_CTRL_REG1, data[0]) ;
//    i2c_writeReg(REG_CTRL_REG1, 0x00) ; /* standby */
//    i2c_writeReg(REG_CTRL_REG1, 0x80) ; /* run */

    data[0] = 0x04 ; /* ODR 12Hz */
    i2c_writeRegs(REG_CTRL_REG2, data, 1) ;
    
    data[0] = 0x40 ; /* PC1 = 0, RES = 1, GSEL = 00 +/-2g */
    i2c_writeRegs(REG_CTRL_REG1, data, 1) ;
    
    data[0] = 0x02 ;
    i2c_writeRegs(REG_DATA_CTRL_REG, data, 1) ;

    data[0] = 0xC0 ;
    i2c_writeRegs(REG_CTRL_REG1, data, 1) ;
//    KXTJ2_run() ;
    i2c_readRegs(REG_CTRL_REG1, data, 1) ; /* for debug */
}

void    KXTJ2_standby(void) 
{
    uint8_t data = 0 ;
    i2c_set_slave_address(I2C_KXTJ2_ADDRESS) ;
    i2c_readRegs(REG_CTRL_REG1, &data, 1) ;
    data = data & (~KXTJ2_PC1) ; /* bit[7] PC1 <= 0 (stand-by) */
    i2c_writeRegs(REG_CTRL_REG1, &data, 1) ;
}

void    KXTJ2_run(void) 
{
    uint8_t data = 0 ;
    i2c_set_slave_address(I2C_KXTJ2_ADDRESS) ;
    i2c_readRegs(REG_CTRL_REG1, &data, 1) ;
    data = data | KXTJ2_PC1 ; /* bit[7] PC1 <= 1 (operate) */
    i2c_writeRegs(REG_CTRL_REG1, &data, 1) ;
}

void    KXTJ2_EnableInterrupt(int mode)  /* 0: disable 1: enable */
{
    int wasRunning = 0 ;
    uint8_t intdata, ctrldata ;
    
    i2c_set_slave_address(I2C_KXTJ2_ADDRESS) ;    
    i2c_readRegs(REG_CTRL_REG1, &ctrldata, 1) ;
    if (ctrldata & KXTJ2_PC1) { /* operation mode */
        KXTJ2_standby() ;
        wasRunning = 1 ;
    }
        
    /* bit[5] : IEN physical interrupt pin enable 0: diable 1: enable */
    /* bit[4] : IEA polarity of physical interrupt pin 0: active low 1: active high */
    /* bit[2] : IEL response of interrupt pin 0: stay until cleared 1: single pulse */
    if (mode) { /* interrupt enable */
        intdata = 0x20 ; /* IEN bit[5] physical interrupt pin enable */
    } else { /* interrupt disable */
        intdata = 0x00 ; /* reset value */   
    }
    i2c_writeRegs(REG_INT_CTRL_REG1, &intdata, 1) ;
    
    if (wasRunning) {
        KXTJ2_run() ;
    }
}
void KXTJ2_ClearInterrupt(void) 
{
    uint8_t dummy ;
    i2c_set_slave_address(I2C_KXTJ2_ADDRESS) ;    
    i2c_readRegs(REG_INT_REL, &dummy, 1) ;
    KXTJ2_INT_ClearInterrupt() ;  /* clear PSoC INT flag, should we do this here? */
}

uint8_t KXTJ2_getODR(void) 
{
    uint8_t odr ;
    i2c_set_slave_address(I2C_KXTJ2_ADDRESS) ;
    i2c_readRegs(REG_DATA_CTRL_REG, &odr, 1) ;
    return( odr ) ;
}

void    KXTJ2_setODR(uint8_t odr) 
{
    i2c_set_slave_address(I2C_KXTJ2_ADDRESS) ;
    i2c_writeRegs(REG_DATA_CTRL_REG, &odr, 1) ;    
}

uint8_t KXTJ2_getID(void)
{
    uint8_t id ;
    i2c_set_slave_address(I2C_KXTJ2_ADDRESS) ;
    i2c_readRegs(REG_WHO_AM_I, &id, 1) ;
    return( id ) ;
}

int16_t KXTJ2_getX(void) 
{
    uint8_t data[2] ;
    int16_t value ;
    i2c_set_slave_address(I2C_KXTJ2_ADDRESS) ;    
    i2c_readRegs(REG_XOUT_L, &data[0], 1) ;
    i2c_readRegs(REG_XOUT_H, &data[1], 1) ;    
    value = (int16_t)((data[1] << 8) | data[0]) ;
    return( value ) ;
}

int16_t KXTJ2_getY(void) 
{
    uint8_t data[2] ;
    int16_t value ;
    i2c_set_slave_address(I2C_KXTJ2_ADDRESS) ;    
    i2c_readRegs(REG_YOUT_L, &data[0], 1) ;
    i2c_readRegs(REG_YOUT_H, &data[1], 1) ;    
    value = (int16_t)((data[1] << 8) | data[0]) ;
    return( value ) ;
}

int16_t KXTJ2_getZ(void) 
{
    uint8_t data[2] ;
    int16_t value ;
    i2c_set_slave_address(I2C_KXTJ2_ADDRESS) ;    
    i2c_readRegs(REG_ZOUT_L, &data[0], 1) ;
    i2c_readRegs(REG_ZOUT_H, &data[1], 1) ;    
    value = (int16_t)((data[1] << 8) | data[0]) ;
    return( value ) ;
}

void    KXTJ2_getAllRawData(int16_t data[]) 
{
    uint8_t bdata[6] ;
    i2c_set_slave_address(I2C_KXTJ2_ADDRESS) ; 
    
    i2c_readRegs(REG_XOUT_L, &bdata[0], 1) ;
    i2c_readRegs(REG_XOUT_H, &bdata[1], 1) ;  
    i2c_readRegs(REG_YOUT_L, &bdata[2], 1) ;
    i2c_readRegs(REG_YOUT_H, &bdata[3], 1) ;      
    i2c_readRegs(REG_ZOUT_L, &bdata[4], 1) ;
    i2c_readRegs(REG_ZOUT_H, &bdata[5], 1) ;  
    
    data[0] = (int16_t)((bdata[1] << 8) | bdata[0]) ;
    data[1] = (int16_t)((bdata[3] << 8) | bdata[2]) ;
    data[2] = (int16_t)((bdata[5] << 8) | bdata[4]) ;    
}

void    KXTJ2_getAllAxis(float data[]) 
{
    int16_t idata[3] ;
    KXTJ2_getAllRawData(idata) ;
    data[0] = 2.0 * (float)idata[0] / 32768.0 ;
    data[1] = 2.0 * (float)idata[1] / 32768.0 ;
    data[2] = 2.0 * (float)idata[2] / 32768.0 ;
}


void KXTJ2_test(int col, int row) 
{
    uint8_t id, ctrl_reg ;
    int16_t x, y, z ;

    id = KXTJ2_getID() ;
    i2c_set_slave_address(I2C_KXTJ2_ADDRESS) ; 
    i2c_readRegs(REG_CTRL_REG1, &ctrl_reg, 1) ;
    x = KXTJ2_getX() ;
    y = KXTJ2_getY() ;
    z = KXTJ2_getZ() ;
//    KXTJ2_ClearInterrupt() ;
    
    sprintf(str, "X:%6d  Y:%6d  Z:%6d\n", x, y, z ) ;

    TFT_BusEnable(1) ;
    TFT_locate(col, row) ;
    TFT_putstr(str) ;
    TFT_BusEnable(0) ;
}
/* [] END OF FILE */
