/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/
#include <project.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "vt100.h"

#define ESC '\033'
static char _str[STR_LENGTH+1] ;
static int _foreground = VT100_BLACK ;
static int _background = VT100_WHITE ;

void initVT100(int fg, int bg) 
{
    setFG(fg) ;
    setBG(bg) ;
    cls() ;
}

void putNL(void)
{
    UART_UartPutCRLF(0) ;
}

void putChar(char c)
{
    UART_UartPutChar(c) ;
}

void putCharAt(int x, int y, char c)
{
    locate(x, y) ;
    putChar(c) ;
}

int getChar(void)
{
    int c ;
    c = UART_UartGetChar() ;
    return(c) ;
}
void putStr(char *str)
{
    char _buf[UART_BUFSIZE+1] ;
    int i = 0 ;
    while((str != 0)&&(*str != 0)) {
        _buf[i++] = *str++ ;
        if (i >= UART_BUFSIZE) {
            _buf[i] = 0 ;
            UART_UartPutString(_buf) ;
            i = 0 ;
        }
    }
    if (i != 0) {
        _buf[i] = 0 ;
        UART_UartPutString(_buf) ;
    }    
}

void putStrAt(int x, int y, char *str)
{
    locate(x, y) ;
    putStr(str) ;
}

int getStr(char str[])
{
    char c ;
    int len = 0 ;
    while((c = UART_UartGetChar()) != 0x0D ) {
        if (c != 0) {
            str[len++] = c ;
            if (len >= STR_LENGTH) {
                break ;
            }
        }
    }
    str[len] = 0 ;
    return( len ) ;
}

/** clear screen */
void cls(void) 
{
    sprintf(_str, "%c[2J", ESC) ;
    putStr(_str) ;
    locate(1,1) ;
}

/** move cursor to (x, y) 
 * @param x start column of the next letter
 * @param y start row of the next letter
 * @note no value checking is performed.
 */
void locate(int x, int y) 
{
    sprintf(_str, "%c[%d;%dH",ESC,y,x) ;
    putStr(_str) ;
}

/** print a line of char 
 * @param c the letter to form the line
 * @param x1 starting column
 * @param y1 starting row
 * @param x2 ending column
 * @param y2 ending row
 */
void line(int x1, int y1, int x2, int y2, char c) 
{
    int x, y, dx, dy, w, h, step ;
    dx = x2 - x1 ;
    dy = y2 - y1 ;
    w = (dx >= 0)? dx : -dx ;
    h = (dy >= 0)? dy : -dy ;
    
    if (!isprint(c)) {
        c = '*' ;
    }

    if (dx == 0) { /* vertical line */
        step = (dy >= 0) ? 1 : -1 ;
        for (y = 0 ; y <= h ; y++) {
            putCharAt(x1, y1 + (step * y), c) ;
        }
    } else if (dy == 0) { /* Horizontal line */
        step = (dx >= 0) ? 1 : -1 ;
        for (x = 0 ; x <= w ; x++) {
            putCharAt(x1 + (step * x), y1, c) ;
        }
    } else {
        if (w >= h) { /* use x as step */
            step = (dx >= 0) ? 1 : -1 ;
            for (x = 0 ; x <= w ; x++ ) {
                putCharAt(x1 + step*x, y1 + ((2*x+1) * dy)/(2 * w), c) ;
            }
        } else { /* use y as step */
            step = (dy >= 0) ? 1 : -1 ;
            for (y = 0 ; y <= h ; y++ ) {
                putCharAt(x1 + ((2*y+1) * dx)/(2*h), y1 + step*y, c) ;
            }
        }
    }
}

/** print a text frame 
 * @param x1 left column
 * @param y1 top row
 * @param x2 right column
 * @param y2 bottom row
*/
/****************************************************
 *  frame(x1, y1, x2, y2)
 *  draw textual frame
 *  (x1,y1)                    (x2,y1)
 *     +--------------------------+
 *     |                          |
 *     +--------------------------+
 *  (x1,y2)                    (x2,y2)
 */
void frame(int x1, int y1, int x2, int y2)
{
    int tmp ;
    if (x1 > x2) {
        tmp = x1 ;
        x1 = x2 ;
        x2 = tmp ;
    }
    if (y1 > y2) {
        tmp = y1 ;
        y1 = y2 ;
        y2 = tmp ;
    }
    putCharAt(x1, y1, '+') ;
    line(x1+1,y1,x2-1,y1, '-') ; 
    putCharAt(x2, y1, '+') ; 
    line(x2,y1+1,x2,y2-1, '|') ; 
    putCharAt(x2, y2, '+') ; 
    line(x2-1,y2,x1+1,y2, '-') ; 
    putCharAt(x1, y2, '+') ; 
    line(x1,y2-1,x1,y1+1, '|') ; 
}
    
/** print a text circle
 * @c the letter to form the circle
 * @param x0 center column
 * @param y1 center row
 * @param r radius
 */
/***************************************************
 *  circle(x, y, r, c) 
 *  Based on Jack Elton Bresenham's
 *  Midpoint circle algorithm.
 *  http://en.wikipedia.org/wiki/Midpoint_circle_algorithm
  */
void circle(int x0, int y0, int r, char c) 
{
    int f = 1 - r ;
    int dFx = 1 ;
    int dFy = -2 * r ;
    int x = 0 ;
    int y = r ;
    
    putCharAt(x0, y0 + r, c) ; 
    putCharAt(x0, y0 - r, c) ; 
    putCharAt(x0 + 2*r, y0, c) ; 
    putCharAt(x0 - 2*r, y0, c) ; 
    
    while(x < y) {
        if (f >= 0) {
            y-- ;
            dFy += 2 ;
            f += dFy ;
        }
        x++ ;
        dFx += 2 ;
        f += dFx ;
        putCharAt(x0 + 2*x, y0 + y, c) ; 
        putCharAt(x0 - 2*x, y0 + y, c) ; 
        putCharAt(x0 + 2*x, y0 - y, c) ; 
        putCharAt(x0 - 2*x, y0 - y, c) ; 
        putCharAt(x0 + 2*y, y0 + x, c) ; 
        putCharAt(x0 - 2*y, y0 + x, c) ; 
        putCharAt(x0 + 2*y, y0 - x, c) ; 
        putCharAt(x0 - 2*y, y0 - x, c) ; 
    }
}

/** set foreground color
 * @param newFG new foreground color
 * @returns previous foreground color
 * @note 0 BLACK
 * @note 1 RED
 * @note 2 GREEN
 * @note 3 YELLOW
 * @note 4 BLUE
 * @note 5 PURPLE
 * @note 6 CIAN
 * @note 7 WHITE
 */
int setFG(int newFG) 
{
    int oldFG = _foreground ;
    sprintf(_str, "%c[3%dm",ESC, newFG) ;
    putStr(_str) ;
    _foreground = newFG ;
    return( oldFG ) ;
}
/** get current foreground color 
 * @returns current foreground color
 */
int getFG(void)
{
    return( _foreground ) ;
}

/** set background color
 * @param newBG new background color
 * @returns previous background color
 */
int setBG(int newBG)
{
    int oldBG = _background ;
    sprintf(_str, "%c[4%dm",ESC, newBG) ;
    putStr(_str) ;
    return( oldBG ) ;
}

/** get current background color 
 * @returns current background color
 */
int getBG(void)
{
    return( _background ) ;
}

/** set foreground color to black */
void black(void)  { setFG(VT100_BLACK) ; }
/** set foreground color to red */
void red(void)    { setFG(VT100_RED) ; }
/** set foreground color to green */
void green(void)  { setFG(VT100_GREEN) ; }
/** set foreground color to yellow */
void yellow(void) { setFG(VT100_YELLOW) ; }
/** set foreground color to blue */
void blue(void)   { setFG(VT100_BLUE) ; }
/** set foreground color to purple */
void purple(void) { setFG(VT100_PURPLE) ; }
/** set foreground color to cian */
void cian(void)   { setFG(VT100_CIAN) ; }
/** set foreground color to white */
void white(void)  { setFG(VT100_WHITE) ; }
    
/* [] END OF FILE */
