/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/
#ifndef _VT100_H_
#define _VT100_H_

#define STR_LENGTH 256
#define UART_BUFSIZE 8
    
/* VT100 Color Numbers */
#define VT100_BLACK   0
#define VT100_RED     1
#define VT100_GREEN   2
#define VT100_YELLOW  3
#define VT100_BLUE    4
#define VT100_PURPLE  5
#define VT100_CIAN    6
#define VT100_WHITE   7
    
void putChar(char c) ;
void putCharAt(int x, int y, char c) ;
int getChar(void) ;
void putStr(char *str) ;
void putStrAt(int x, int y, char *str) ;
int getStr(char *str) ;
void putNL(void) ;

void initVT100(int fg, int bg) ;

/** clear screen */
void cls(void) ;

/** move cursor to (x, y) 
 * @param x start column of the next letter
 * @param y start row of the next letter
 * @note no value checking is performed.
 */
void locate(int x, int y) ;

/** print a line of char 
 * @param c the letter to form the line
 * @param x1 starting column
 * @param y1 starting row
 * @param x2 ending column
 * @param y2 ending row
 */
void line(int x1, int y1, int x2, int y2, char c) ;

/** print a text frame 
 * @param x1 left column
 * @param y1 top row
 * @param x2 right column
 * @param y2 bottom row
*/
void frame(int x1, int y1, int x2, int y2) ;
    
/** print a text circle
 * @c the letter to form the circle
 * @param x0 center column
 * @param y1 center row
 * @param r radius
 */
void circle(int x0, int y0, int r, char c) ;

/** set foreground color
 * @param newFG new foreground color
 * @returns previous foreground color
 * @note 0 BLACK
 * @note 1 RED
 * @note 2 GREEN
 * @note 3 YELLOW
 * @note 4 BLUE
 * @note 5 PURPLE
 * @note 6 CIAN
 * @note 7 WHITE
 */
int setFG(int newFG) ;
    
/** get current foreground color 
 * @returns current foreground color
 */
int getFG(void) ;

/** set background color
 * @param newBG new background color
 * @returns previous background color
 */
int setBG(int newBG) ;
    
/** get current background color 
 * @returns current background color
 */
int getBG(void) ;

/** set foreground color to black */
void black(void) ;
/** set foreground color to red */
void red(void) ;
/** set foreground color to green */
void green(void) ;
/** set foreground color to yellow */
void yellow(void) ;
/** set foreground color to blue */
void blue(void) ;
/** set foreground color to purple */
void purple(void) ;
/** set foreground color to cian */
void cian(void) ;
/** set foreground color to white */
void white(void) ;
    
#endif /* _VT100_H_ */
/* [] END OF FILE */
